import { Toolbar, Toast, DateHelper, CSSHelper } from '../../build/gantt.module.js';
import '../../lib/Scheduler/widget/UndoRedo.js';
/**
 * @module GanttToolbar
 */

/**
 * @extends Core/widget/Toolbar
 */
export default class GanttToolbar extends Toolbar {
    // Factoryable type name
    static get type() {
        return 'gantttoolbar';
    }

    static get $name() {
        return 'GanttToolbar';
    }

    construct(config) {
        const
            me      = this,
            gantt   = me.gantt = me.owner = config.gantt,
            project = gantt.project;

        project.on({
            load                : me.updateStartDateField,
            propagationComplete : me.updateStartDateField,
            thisObj             : me
        });

        const stm = project.stm;

        stm.on({
            recordingstop : me.updateUndoRedoButtons,
            restoringstop : me.updateUndoRedoButtons,
            queueReset    : me.updateUndoRedoButtons,
            thisObj       : me
        });

        super.construct(config);

        me.styleNode = document.createElement('style');
        document.head.appendChild(me.styleNode);
    }

    static get defaultConfig() {
        return {
            items : [
                {
                    type  : 'buttonGroup',
                    items : [
                        {
                            type     : 'button',
                            color    : 'b-green',
                            ref      : 'addTaskButton',
                            icon     : 'b-fa b-fa-plus',
                            text     : 'Create',
                            tooltip  : 'Create new task',
                            onAction: 'up.onAddTaskClick',
                            hidden: false
                        }
                    ]
                },
                {
                    type  : 'buttonGroup',
                    items : [
                        {
                            type     : 'button',
                            color    : 'b-blue',
                            ref      : 'editTaskButton',
                            icon     : 'b-fa b-fa-pen',
                            text     : 'Edit',
                            tooltip  : 'Edit selected task',
                            onAction: 'up.onEditTaskClick',
                            hidden: false
                        },
                        //{
                        //    type     : 'button',
                        //    color    : 'b-blue',
                        //    ref      : 'undoBtn',
                        //    icon     : 'b-icon b-fa b-fa-undo',
                        //    tooltip  : 'Undo',
                        //    disabled : true,
                        //    width    : '2em',
                        //    onAction: 'up.onUndoClick',
                        //    //hidden: false
                        //},
                        //{
                        //    type     : 'button',
                        //    color    : 'b-blue',
                        //    ref      : 'redoBtn',
                        //    icon     : 'b-icon b-fa b-fa-redo',
                        //    tooltip  : 'Redo',
                        //    disabled : true,
                        //    width    : '2em',
                        //    onAction: 'up.onRedoClick',
                        //    //hidden: false
                        //}
                    ]
                },
                {
                    ref: 'undoRedo',
                    type: 'undoredo',
                    hidden: false,
                    items: {
                        transactionsCombo: null
                    }
                },
                {
                    type  : 'buttonGroup',
                    items : [
                        {
                            type     : 'button',
                            color    : 'b-blue',
                            ref      : 'expandAllButton',
                            icon     : 'b-fa b-fa-angle-double-down',
                            tooltip  : 'Expand all',
                            onAction : 'up.onExpandAllClick'
                        },
                        {
                            type     : 'button',
                            color    : 'b-blue',
                            ref      : 'collapseAllButton',
                            icon     : 'b-fa b-fa-angle-double-up',
                            tooltip  : 'Collapse all',
                            onAction : 'up.onCollapseAllClick'
                        }
                    ]
                },
                {
                    type  : 'buttonGroup',
                    items : [
                        {
                            type     : 'button',
                            color    : 'b-blue',
                            ref      : 'zoomInButton',
                            icon     : 'b-fa b-fa-search-plus',
                            tooltip  : 'Zoom in',
                            onAction : 'up.onZoomInClick'
                        },
                        {
                            type     : 'button',
                            color    : 'b-blue',
                            ref      : 'zoomOutButton',
                            icon     : 'b-fa b-fa-search-minus',
                            tooltip  : 'Zoom out',
                            onAction : 'up.onZoomOutClick'
                        },
                        {
                            type     : 'button',
                            color    : 'b-blue',
                            ref      : 'zoomToFitButton',
                            icon     : 'b-fa b-fa-compress-arrows-alt',
                            tooltip  : 'Zoom to fit',
                            onAction : 'up.onZoomToFitClick'
                        },
                        {
                            type     : 'button',
                            color    : 'b-blue',
                            ref      : 'previousButton',
                            icon     : 'b-fa b-fa-angle-left',
                            tooltip  : 'Previous time span',
                            onAction : 'up.onShiftPreviousClick'
                        },
                        {
                            type     : 'button',
                            color    : 'b-blue',
                            ref      : 'nextButton',
                            icon     : 'b-fa b-fa-angle-right',
                            tooltip  : 'Next time span',
                            onAction : 'up.onShiftNextClick'
                        }
                    ]
                },                
                {
                    type: 'buttonGroup',
                    items: [
                        {
                            type: 'button',
                            color: 'b-blue',
                            ref: 'indentBtn',
                            icon: 'b-icon b-fa b-fa-indent',
                            tooltip: 'Indent',
                            width: '2em',
                            onAction: 'up.onIndentClick',
                            hidden: false
                        },
                        {
                            type: 'button',
                            color: 'b-blue',
                            ref: 'outdentBtn',
                            icon: 'b-icon b-fa b-fa-outdent',
                            tooltip: 'Outdent',
                            width: '2em',
                            onAction: 'up.onOutdentClick',
                            hidden: false
                        },
                        //{
                        //    type: 'button',
                        //    color: 'b-blue',
                        //    text: 'predecessor',
                        //    onAction: 'up.onAddPredecessorClick',
                        //    hidden: false
                        //},
                        //{
                        //    type: 'button',
                        //    color: 'b-blue',
                        //    text: 'successor',
                        //    onAction: 'up.onAddSuccessorClick',
                        //    hidden: false
                        //}
                    ]
                },
                {
                    type                 : 'textfield',
                    ref                  : 'filterByName',
                    cls                  : 'filter-by-name',
                    flex                 : '1 1 12.5em',
                    // Label used for material, hidden in other themes
                    label                : 'Find tasks by name',
                    // Placeholder for others
                    placeholder          : 'Find tasks by name',
                    clearable            : true,
                    keyStrokeChangeDelay : 100,
                    triggers             : {
                        filter : {
                            align : 'end',
                            cls   : 'b-fa b-fa-filter'
                        }
                    },
                    onChange : 'up.onFilterChange'
                },
                {
                    type: 'button',
                    ref: 'featuresButton',
                    icon: 'b-fa b-fa-tasks',
                    text: 'Settings',
                    tooltip: 'Toggle features',
                    menu: {
                        onItem: 'up.onFeaturesClick',
                        onBeforeShow: 'up.onFeaturesShow',
                        // "checked" is set to a boolean value to display a checkbox for menu items. No matter if it is true or false.
                        // The real value is set dynamically depending on the "disabled" config of the feature it is bound to.
                        items: [
                            {
                                text: 'UI settings',
                                icon: 'b-fa-sliders-h',
                                menu: {
                                    cls: 'settings-menu',
                                    layoutStyle: {
                                        flexDirection: 'column'
                                    },
                                    onBeforeShow: 'up.onSettingsShow',
                                    defaults: {
                                        type: 'slider',
                                        showValue: true
                                    },
                                    items: [
                                        {
                                            ref: 'rowHeight',
                                            text: 'Row height',
                                            min: 30,
                                            max: 70,
                                            onInput: 'up.onRowHeightChange'
                                        },
                                        {
                                            ref: 'barMargin',
                                            text: 'Bar margin',
                                            min: 0,
                                            max: 10,
                                            onInput: 'up.onBarMarginChange'
                                        },
                                        {
                                            ref: 'duration',
                                            text: 'Animation duration',
                                            min: 0,
                                            max: 2000,
                                            step: 100,
                                            onInput: 'up.onAnimationDurationChange'
                                        },
                                        {
                                            ref: 'radius',
                                            text: 'Dependency radius',
                                            min: 0,
                                            max: 10,
                                            onInput: 'up.onDependencyRadiusChange'
                                        }
                                    ]
                                }
                            },
                            {
                                text: 'Draw dependencies',
                                feature: 'dependencies',
                                checked: true
                            },
                            {
                                text: 'Task labels',
                                feature: 'labels',
                                checked: true
                            },
                            {
                                text: 'Critical paths',
                                feature: 'criticalPaths',
                                tooltip: 'Highlight critical paths',
                                checked: false
                            },
                            {
                                text: 'Project lines',
                                feature: 'projectLines',
                                checked: true
                            },
                            {
                                text: 'Highlight non-working time',
                                feature: 'nonWorkingTime',
                                checked: true
                            },
                            //{
                            //    text: 'Enable cell editing',
                            //    feature: 'cellEdit',
                            //    checked: false,
                            //},
                            {
                                text: 'Show column lines',
                                feature: 'columnLines',
                                checked: true
                            },
                            {
                                text: 'Show baselines',
                                feature: 'baselines',
                                checked: false
                            },
                            {
                                text: 'Show rollups',
                                feature: 'rollups',
                                checked: false
                            },
                            {
                                text: 'Show progress line',
                                feature: 'progressLine',
                                checked: false
                            },
                            {
                                text: 'Show parent area',
                                feature: 'parentArea',
                                checked: false
                            },
                            {
                                text: 'Stretch tasks to fill ticks',
                                toggleConfig: 'fillTicks',
                                checked: false
                            },
                            {
                                text: 'Hide schedule',
                                cls: 'b-separator',
                                subGrid: 'normal',
                                checked: false
                            }
                        ]
                    }
                }
            ]
        };
    }

    updateUndoRedoButtons() {
        const
            { stm }              = this.gantt.project,
            { undoBtn, redoBtn } = this.widgetMap,
            redoCount            = stm.length - stm.position;

        undoBtn.badge = stm.position || '';
        redoBtn.badge = redoCount || '';

        undoBtn.disabled = !stm.canUndo;
        redoBtn.disabled = !stm.canRedo;
    }

    setAnimationDuration(value) {
        const
            me      = this,
            cssText = `.b-animating .b-gantt-task-wrap { transition-duration: ${value / 1000}s !important; }`;

        me.gantt.transitionDuration = value;

        if (me.transitionRule) {
            me.transitionRule.cssText = cssText;
        }
        else {
            me.transitionRule = CSSHelper.insertRule(cssText);
        }
    }

    updateStartDateField() {
        //const startDateField = this.widgetMap.startDateField;

        //startDateField.value = this.gantt.project.startDate;

        //// This handler is called on project.load/propagationComplete, so now we have the
        //// initial start date. Prior to this time, the empty (default) value would be
        //// flagged as invalid.
        //startDateField.required = true;
    }

    // region controller methods

    async onAddTaskClick() {
        const
            { gantt } = this,
            added = gantt.taskStore.rootNode.appendChild({ name : 'New task', duration : 1 });

        // run propagation to calculate new task fields
        await gantt.project.commitAsync();

        // scroll to the added task
        await gantt.scrollRowIntoView(added);

        gantt.features.cellEdit.startEditing({
            record : added,
            field  : 'name'
        });
    }

    onEditTaskClick() {
        const { gantt } = this;

        if (gantt.selectedRecord) {
            gantt.editTask(gantt.selectedRecord);
        }
        else {
            Toast.show('First select the task you want to edit');
        }
    }

    onIndentClick() {
        const { gantt } = this;
        if (gantt.selectedRecords.length > 0) {
            gantt.indent(gantt.selectedRecords);
        }
        else {
            Toast.show('First select the task you want to indent');
        }
    }

    onOutdentClick() {
        const { gantt } = this;
        if (gantt.selectedRecords.length > 0) {
            gantt.outdent(gantt.selectedRecords);
        }
        else {
            Toast.show('First select the task you want to outdent');
        }
    }

    onAddPredecessorClick() {
        const { gantt } = this;
        if (gantt.selectedRecords.length > 0) {
            gantt.addPredecessor(gantt.selectedRecords[0]);
        } else {
            Toast.show('Please select a task first');
        }
    }

    onAddSuccessorClick() {
        const { gantt } = this;
        if (gantt.selectedRecords.length > 0) {
            gantt.addSuccessor(gantt.selectedRecords[0]);
        } else {
            Toast.show('Please a select a task first');
        }
    }

    onExpandAllClick() {
        this.gantt.expandAll();
    }

    onCollapseAllClick() {
        this.gantt.collapseAll();
    }

    onZoomInClick() {
        this.gantt.zoomIn();
    }

    onZoomOutClick() {
        this.gantt.zoomOut();
    }

    onZoomToFitClick() {
        this.gantt.zoomToFit({
            leftMargin  : 50,
            rightMargin : 50
        });
    }

    onShiftPreviousClick() {
        this.gantt.shiftPrevious();
    }

    onShiftNextClick() {
        this.gantt.shiftNext();
    }

    onStartDateChange({ value, oldValue }) {
        if (!oldValue) { // ignore initial set
            return;
        }

        this.gantt.startDate = DateHelper.add(value, -1, 'week');

        this.gantt.project.setStartDate(value);
    }

    onFilterChange({ value }) {
        if (value === '') {
            this.gantt.taskStore.clearFilters();
        }
        else {
            value = value.replace(/[.*+?^${}()|[\]\\]/g, '\\$&');

            this.gantt.taskStore.filter(task => task.name && task.name.match(new RegExp(value, 'i')));
        }
    }

    //onFeaturesClick({ source : item }) {
    //    const { gantt } = this;

    //    if (item.feature) {
    //        const feature = gantt.features[item.feature];
    //        feature.disabled = !feature.disabled;
    //    }
    //    else if (item.subGrid) {
    //        const subGrid = gantt.subGrids[item.subGrid];
    //        subGrid.collapsed = !subGrid.collapsed;
    //    }
    //}

    //onFeaturesShow({ source : menu }) {
    //    const { gantt } = this;

    //    menu.items.map(item => {
    //        const { feature } = item;

    //        if (feature) {
    //            // a feature might be not presented in the gantt
    //            // (the code is shared between "advanced" and "php" demos which use a bit different set of features)
    //            if (gantt.features[feature]) {
    //                item.checked = !gantt.features[feature].disabled;
    //            }
    //            // hide not existing features
    //            else {
    //                item.hide();
    //            }
    //        }
    //        else {
    //            item.checked = gantt.subGrids[item.subGrid].collapsed;
    //        }
    //    });
    //}

    //onSettingsShow({ source : menu }) {
    //    const { gantt } = this,
    //        { widgetMap } = menu;

    //    widgetMap.rowHeight.value = gantt.rowHeight;
    //    widgetMap.barMargin.value = gantt.barMargin;
    //    widgetMap.barMargin.max = (gantt.rowHeight / 2) - 5;
    //    widgetMap.duration.value = gantt.transitionDuration;
    //}

    //onSettingsRowHeightChange({ value }) {
    //    this.gantt.rowHeight = value;
    //    this.widgetMap.settingsButton.menu.widgetMap.barMargin.max = (value / 2) - 5;
    //}

    //onSettingsMarginChange({ value }) {
    //    this.gantt.barMargin = value;
    //}

    //onSettingsDurationChange({ value }) {
    //    this.gantt.transitionDuration = value;
    //    this.styleNode.innerHTML = `.b-animating .b-gantt-task-wrap { transition-duration: ${value / 1000}s !important; }`;
    //}

    onCriticalPathsClick({ source }) {
        this.gantt.features.criticalPaths.disabled = !source.pressed;
    }

    //onUndoClick() {
    //    this.gantt.project.stm.canUndo && this.gantt.project.stm.undo();
    //}

    //onRedoClick() {
    //    this.gantt.project.stm.canRedo && this.gantt.project.stm.redo();
    //}

    onFeaturesClick({ source: item }) {
        const { gantt } = this;

        if (item.feature) {
            const feature = gantt.features[item.feature];
            feature.disabled = !feature.disabled;
        }
        else if (item.subGrid) {
            const subGrid = gantt.subGrids[item.subGrid];
            subGrid.collapsed = !subGrid.collapsed;
        }
        else if (item.toggleConfig) {
            gantt[item.toggleConfig] = item.checked;
        }
    }

    onFeaturesShow({ source: menu }) {
        const { gantt } = this;

        menu.items.map(item => {
            const { feature } = item;

            if (feature) {
                // a feature might be not presented in the gantt
                // (the code is shared between "advanced" and "php" demos which use a bit different set of features)
                if (gantt.features[feature]) {
                    item.checked = !gantt.features[feature].disabled;
                }
                // hide not existing features
                else {
                    item.hide();
                }
            }
            else if (item.subGrid) {
                item.checked = gantt.subGrids[item.subGrid].collapsed;
            }
        });
    }

    onSettingsShow({ source: menu }) {
        const
            { gantt } = this,
            { rowHeight, barMargin, duration, radius } = menu.widgetMap;

        rowHeight.value = gantt.rowHeight;
        barMargin.value = gantt.barMargin;
        barMargin.max = (gantt.rowHeight / 2) - 5;
        duration.value = gantt.transitionDuration;
        radius.value = gantt.features.dependencies.radius ?? 0;
    }

    onRowHeightChange({ value, source }) {
        this.gantt.rowHeight = value;
        source.owner.widgetMap.barMargin.max = (value / 2) - 5;
    }

    onBarMarginChange({ value }) {
        this.gantt.barMargin = value;
    }

    onAnimationDurationChange({ value }) {
        this.gantt.transitionDuration = value;
        this.styleNode.innerHTML = `.b-animating .b-gantt-task-wrap { transition-duration: ${value / 1000}s !important; }`;
    }

    onDependencyRadiusChange({ value }) {
        this.gantt.features.dependencies.radius = value;
    }

    // endregion
};

// Register this widget type with its Factory
GanttToolbar.initClass();
